#include "timetableengine.h"

#include <QDate>
#include <QTime>
#include <QProcess>
#include <QStringList>
#include <QFile>
 
#include "plasma/datacontainer.h"
 
TimetableEngine::TimetableEngine(QObject* parent, const QVariantList& args)
    : Plasma::DataEngine(parent, args), mLoadedSource(false), mName(0)
{
    // We ignore any arguments - data engines do not have much use for them
    Q_UNUSED(args)
 
    // This prevents applets from setting an unnecessarily high
    // update interval and using too much CPU.
    // In the case of a clock that only has second precision,
    // a third of a second should be more than enough.
    setMinimumPollingInterval(1000);

}

void TimetableEngine::init()
{
    connect( Solid::Networking::notifier(), SIGNAL( statusChanged( Solid::Networking::Status ) ),
             this, SLOT( networkStatusChanged( Solid::Networking::Status ) ) );
}

void TimetableEngine::networkStatusChanged( Solid::Networking::Status status )
{
    if ( ( status == Solid::Networking::Connected || status == Solid::Networking::Unknown ) &&
         !mLoadedSource && mName!="" ) {
        sourceRequestEvent( mName );
    }
}
 
bool TimetableEngine::sourceRequestEvent(const QString &name)
{
    QString command;
    mName=name;
    mLoadedSource=false;

    if(Solid::Networking::Connected == Solid::Networking::status() || Solid::Networking::Unknown == Solid::Networking::status()) {
	//Descarrega horaris
	QProcess* process = new QProcess(this);
	if(name.split(";;")[0]=="RENFE")
		command = "wget -O /tmp/.hrenfe.html http://horarios.renfe.es/cer/hjcer310.jsp?&nucleo=50&o=" + getRenfeCode(name.split(";;")[1]) + "&d=" + getRenfeCode(name.split(";;")[2]) + "&tc=DIA&td=D&df=" + (QDate::currentDate()).toString("yyyyMMdd") + "&th=1&ho=00&i=0&cp=NO&TXTInfo=";
	else if(name.split(";;")[0]=="FGC")
		command = QString("wget -O /tmp/.hfgc.html http://www.fgc.cat/cat/cercador.php?") + "estacio_origen=" + getFGCCode(name.split(";;")[1]) + "&estacio_desti=" + getFGCCode(name.split(";;")[2]) + "&dia_viatge=" + (QDate::currentDate()).toString("dd") + "&mes_viatge=" + (QDate::currentDate()).toString("MM") + "&any_viatge=" + (QDate::currentDate()).toString("yyyy") + "&tipus_hora=franja&hfd=" + (QTime::currentTime()).toString("HH") + "&hfh=00";
	else return(false);

	emit isBusy( true );
	process->start(command);
	process->waitForFinished();
	delete process;
	emit isBusy( false );
	
	mLoadedSource=true;
	return updateSourceEvent(name);
    } else return false;
}
 
bool TimetableEngine::updateSourceEvent(const QString &name)
{
    if(mLoadedSource){
	QStringList horaris;
	QString lhoraris;

	if(name.split(";;")[0]=="RENFE"){
		horaris = getRenfe(name);
		lhoraris=horaris.join("\n");
		setData(name, I18N_NOOP("RENFE Hora Sortida"), horaris);
	} else if (name.split(";;")[0]=="FGC"){
		horaris = getFGC(name);
		lhoraris=horaris.join("\n");
		setData(name, I18N_NOOP("FGC Hora Sortida"), horaris);
	}
	return true;
    } else return false;
}

QStringList TimetableEngine::getFGC(const QString &name)
{
    //Parsejar HTML
    QString line;
    QStringList horaris;
    QFile file("/tmp/.hfgc.html");
    if (file.open(QFile::ReadOnly)) {
	char buf[1024];
	int n=0;

	while(!line.contains("trajecte_que_mostrem=")) {
		file.readLine(buf, sizeof(buf));
		line=buf;
		if(line.contains("</html>")) return horaris;
	}

	do{
		QString hora = (line.split("class=\"hora\">")[1]).split("<")[0];
		QTime hora_sortida( (hora.split(":")[0]).toInt(), (hora.split(":")[1]).toInt() );

		if(hora_sortida>QTime::currentTime() && n<5){
			horaris.append(hora);
			n++;
		}
		file.readLine(buf, sizeof(buf));
		line=buf;
	} while(line.contains("trajecte_que_mostrem="));

	if(horaris.size()==0) horaris.append("Not Found");
	file.close();
    } else horaris.append("Not Found");
    return(horaris);
}

QStringList TimetableEngine::getRenfe(const QString &name)
{
    //Parsejar HTML
    QString line;
    QStringList horaris;
    QFile file("/tmp/.hrenfe.html");
    if (file.open(QFile::ReadOnly)) {
	char buf[1024];
	int n=0;
	bool final=false;

	while(n<4){
		file.readLine(buf, sizeof(buf));
		line=buf;
		if(line.contains("<table")) n++;
		else if(line.contains("Datos no disponibles.")){
			horaris.append("Not Found");
			return(horaris);
		}
	}

	n=0;
	while(n<3){
		file.readLine(buf, sizeof(buf));
		line=buf;
		if(line.contains("<tr>")) n++;
	}
	
	n=0;
	while(!final){
		while(!line.contains("<td")){ 
			file.readLine(buf, sizeof(buf));
			line=buf;
		}
		if(!line.contains("<br>")){
			QString hora = (((line.split(">"))[1]).split("<"))[0];

			QTime hora_sortida( ((hora.split("."))[0]).toInt(), ((hora.split("."))[1]).toInt() );

			if(hora_sortida>QTime::currentTime()&&n<5) {
				horaris.append(hora);
				n++; //Limit horaris
			}
			while(!line.contains("<tr>")){ 
				file.readLine(buf, sizeof(buf));
				line=buf;
			}
		} else
		    final = true;
	}
	if(horaris.size()==0) horaris.append("Not Found");
	file.close();
    } else horaris.append("Not Found");
    return horaris;
}

QString TimetableEngine::getRenfeCode(const QString &name){
    if(name=="Manresa") return QString("78600");
    else if(name=="Barcelona-Sants") return QString("71801");
    else if(name=="Plaça de Catalunya") return QString("78805");
    else if(name=="Barcelona-Passeig de Gracia") return QString("71802");
    else if(name=="Mataro") return QString("79500");
    else if(name=="Barcelona-Clot-Arago") return QString("79009");
    else if(name=="Arc de Triumf") return QString("78804");
    else if(name=="Sant Andreu Arenal") return QString("78802");
    else if(name=="L'Hospitalet de Llobregat") return QString("72305");
    else if(name=="Vilanova i la Geltru") return QString("71700");
    else if(name=="Cerdanyola del Valles") return QString("78706");
    else if(name=="Castelldefels") return QString("71705");
    else if(name=="Sant Feliu de Llobregat") return QString("72301");
    else if(name=="Terrassa") return QString("78700");
    else if(name=="Premia de Mar") return QString("79409");
    else if(name=="Sitges") return QString("71701");
    else if(name=="Granollers-Centre") return QString("79100");
    else if(name=="Sabadell-Centre") return QString("78704");
    else if(name=="Aeroport") return QString("72400");
    else if(name=="Cornella") return QString("72303");
    else if(name=="Vilassar de Mar") return QString("79410");
    else if(name=="Sabadell Nord") return QString("78709");
    else if(name=="Gava") return QString("71706");
    else if(name=="El Prat de Llobregat") return QString("71707");
    else if(name=="Barcelona-Sant Andreu Comtal") return QString("79004");
    else if(name=="Molins de Rei") return QString("72300");
    else if(name=="Mollet-Sant Fost") return QString("79006");
    else if(name=="Martorell") return QString("72209");
    else if(name=="Vilafranca del Penedes") return QString("72204");
    else if(name=="El Masnou") return QString("79407");
    else if(name=="Torre el Baro") return QString("78801");
    else if(name=="Ocata") return QString("79408");
    else if(name=="Sant Joan Despi") return QString("72302");
    else if(name=="Calella") return QString("79603");
    else if(name=="Sant Adria de Besos") return QString("79403");
    else if(name=="Barbera del Valles") return QString("78705");
    else if(name=="Vic") return QString("77109");
    else if(name=="Viladecans") return QString("71709");
    else if(name=="Arenys de Mar") return QString("79600");
    else if(name=="Cerdanyola Universitat") return QString("72503");
    else if(name=="Sabadell Sud") return QString("78703");
    else if(name=="Montmelo") return QString("79007");
    else if(name=="Sant Celoni") return QString("77109");
    else if(name=="Bellvitge") return QString("71708");
    else if(name=="Montgat") return QString("79405");
    else if(name=="La Llagosta") return QString("79011");
    else if(name=="Mollet-Santa Rosa") return QString("77004");
    else if(name=="Cardedeu") return QString("79101");
    else if(name=="Les Franqueses-Granollers Nord") return QString("79109");
    else if(name=="Granollers-Canovelles") return QString("77006");
    else if(name=="Canet de Mar") return QString("79601");
    else if(name=="Pineda de Mar") return QString("79604");
    else if(name=="Parets del Valles") return QString("77005");
    else if(name=="Malgrat de Mar") return QString("79605");
    else if(name=="Montcada i Reixac-Manresa") return QString("78708");
    else if(name=="Cubelles") return QString("71604");
    else if(name=="Cabrera de Mar-Vilassar de Mar") return QString("79412");
    else if(name=="La Garriga") return QString("77102");
    else if(name=="Llinars del Valles") return QString("79102");
    else if(name=="Sant Sadurni d\'Anoia") return QString("72207");
    else if(name=="Sant Andreu de Llavaneres") return QString("79501");
    else if(name=="Gelida") return QString("72208");
    else if(name=="Caldes d\'Estrac") return QString("79502");
    else if(name=="Platja de Castelldefels") return QString("71704");
    else if(name=="Sant Pol de Mar") return QString("79602");
    else if(name=="Montgat-Nord") return QString("79406");
    else if(name=="Montcada i Reixac-Santa Maria") return QString("78707");
    else if(name=="Santa Perpetua de Mogoda") return QString("77003");
    else if(name=="Castellbisbal") return QString("72210");
    else if(name=="El Papiol") return QString("72211");
    else if(name=="Barcelona-Estacio de França") return QString("79400");
    else if(name=="Sant Vicenç de Castellet") return QString("78604");
    else if(name=="Sant Cugat del Valles") return QString("72502");
    else if(name=="Montcada-Bifurcacio") return QString("78800");
    else if(name=="Montcada i Reixac") return QString("79005");
    else if(name=="Palautordera") return QString("79103");
    else if(name=="Montcada-Ripollet") return QString("77002");
    else if(name=="Tordera") return QString("79607");
    else if(name=="Rubi") return QString("72501");
    else if(name=="Les Franqueses del Valles") return QString("77100");
    else if(name=="Centelles") return QString("77105");
    else if(name=="Santa Susanna") return QString("79608");
    else if(name=="Manlleu") return QString("77110");
    else if(name=="Torello") return QString("77111");
    else if(name=="Balenya-Tona-Seva") return QString("77107");
    else if(name=="Els Monjos") return QString("72203");
    else if(name=="Garraf") return QString("71703");
    else if(name=="Sant Marti de Centelles") return QString("77104");
    else if(name=="Balenya-Els Hostalets") return QString("77106");
    else if(name=="La Granada") return QString("72205");
    else if(name=="Gualba") return QString("79105");
    else if(name=="Figaro") return QString("77103");
    else if(name=="Sant Quirze de Besora-Montesquiu") return QString("77113");
    else if(name=="Vacarisses") return QString("78606");
    else if(name=="Vacarisses-Torreblanca") return QString("78607");
    else if(name=="Viladecavalls") return QString("78609");
    else if(name=="Lavern-Subirats") return QString("72206");
    else if(name=="Sant Miquel de Gonteres") return QString("78609");
    else if(name=="Castellbell i el Vilar-Monistrol de Montserrat") return QString("78605");
    else return QString("Error");
}

QString TimetableEngine::getFGCCode(const QString &name){
    if(name=="Manresa-Baixador") return QString("61");
    else if(name=="Manresa-Alta") return QString("60");
    else if(name=="Manresa-Viladordis") return QString("59");
    else if(name=="Abrera") return QString("51");
    else if(name=="Almeda") return QString("37");
    else if(name=="Av. Tibidabo") return QString("30");
    else if(name=="Baixador de Vallvidrera") return QString("11");
    else if(name=="Bellaterra") return QString("22");
    else if(name=="Can Parellada") return QString("63");
    else if(name=="Can Ros") return QString("44");
    else if(name=="Capellades") return QString("66");
    else if(name=="Castellbell i el Vilar") return QString("57");
    else if(name=="Colonia Guell") return QString("41");
    else if(name=="Cornella Riera") return QString("38");
    else if(name=="El Palau") return QString("48");
    else if(name=="El Putxet") return QString("29");
    else if(name=="Esparreguera") return QString("74");
    else if(name=="Europa | Fira") return QString("290");
    else if(name=="Gelida-Inferior") return QString("94");
    else if(name=="Gelida-Superior") return QString("95");
    else if(name=="Gornal") return QString("34");
    else if(name=="Gracia") return QString("3");
    else if(name=="Hospital General") return QString("17");
    else if(name=="Igualada") return QString("69");
    else if(name=="Ildefons Cerda") return QString("33");
    else if(name=="La Beguda") return QString("54");
    else if(name=="La Bonanova") return QString("6");
    else if(name=="La Floresta") return QString("13");
    else if(name=="La Pobla de Claramunt") return QString("68");
    else if(name=="Les Fonts") return QString("19");
    else if(name=="Les Planes") return QString("12");
    else if(name=="Les Tres Torres") return QString("7");
    else if(name=="L\'Hospitalet-Av. Carrilet") return QString("36");
    else if(name=="Magoria-La Campana") return QString("32");
    else if(name=="Martorell Central") return QString("291");
    else if(name=="Martorell Enllaç") return QString("50");
    else if(name=="Martorell Vila") return QString("49");
    else if(name=="Masquefa") return QString("62");
    else if(name=="Mira-sol") return QString("16");
    else if(name=="Moli Nou-Ciutat Cooperativa") return QString("40");
    else if(name=="Monistrol de M (cremallera)") return QString("70");
    else if(name=="Monistrol de Montserrat") return QString("56");
    else if(name=="Monistrol Vila") return QString("71");
    else if(name=="Montserrat") return QString("72");
    else if(name=="Montserrat-Aeri") return QString("55");
    else if(name=="Muntaner") return QString("5");
    else if(name=="Olesa de Montserrat (aeri)") return QString("73");
    else if(name=="Olesa de Montserrat") return QString("52");
    else if(name=="Padua") return QString("28");
    else if(name=="Palleja") return QString("46");
    else if(name=="Peu del Funicular") return QString("10");
    else if(name=="Piera") return QString("65");
    else if(name=="Pl. Catalunya") return QString("1");
    else if(name=="Pl. Espanya") return QString("31");
    else if(name=="Pl. Molina") return QString("27");
    else if(name=="Provença") return QString("2");
    else if(name=="Quatre Camins") return QString("45");
    else if(name=="Reina Elisenda") return QString("9");
    else if(name=="Rubi") return QString("18");
    else if(name=="Sabadell - Estacio") return QString("25");
    else if(name=="Sabadell - Rambla") return QString("26");
    else if(name=="Sant Andreu de la Barca") return QString("47");
    else if(name=="Sant Boi") return QString("39");
    else if(name=="Sant Cugat") return QString("15");
    else if(name=="Sant Esteve Sesrovires") return QString("53");
    else if(name=="Sant Gervasi") return QString("4");
    else if(name=="Sant Joan") return QString("21");
    else if(name=="Sant Joan-Inferior") return QString("90");
    else if(name=="Sant Joan-Superior") return QString("91");
    else if(name=="Sant Josep") return QString("35");
    else if(name=="Sant Quirze") return QString("24");
    else if(name=="Sant Vicenç dels Horts") return QString("43");
    else if(name=="Sant Vicenç-CastellGali") return QString("58");
    else if(name=="Santa Coloma de Cervello") return QString("42");
    else if(name=="Santa Cova-Inferior") return QString("92");
    else if(name=="Santa Cova-Superior") return QString("93");
    else if(name=="Sarria") return QString("8");
    else if(name=="Terrassa - Rambla") return QString("20");
    else if(name=="Universitat Autonoma") return QString("23");
    else if(name=="Vallbona d\'Anoia") return QString("64");
    else if(name=="Valldoreix") return QString("14");
    else if(name=="Vallvidrera Inferior") return QString("75");
    else if(name=="Vallvidrera Superior") return QString("76");
    else if(name=="Vilanova del Cami") return QString("67");
    else return QString("Error");
}
 
// This does the magic that allows Plasma to load
// this plugin.  The first argument must match
// the X-Plasma-EngineName in the .desktop file.
K_EXPORT_PLASMA_DATAENGINE(timetable, TimetableEngine)
 
// this is needed since TimetableEngine is a QObject
#include "timetableengine.moc"