#ifndef TIMETABLE_ENGINE_H
#define TIMETABLE_ENGINE_H
 
// We need the DataEngine header, since we are inheriting it
#include <Plasma/DataEngine>
#include <solid/networking.h>
 
/**
 * This engine provides the current date and time for a given
 * timezone.
 *
 * "Local" is a special source that is an alias for the current
 * timezone.
 */
class TimetableEngine : public Plasma::DataEngine
{
    // required since Plasma::DataEngine inherits QObject
    Q_OBJECT
 
    public:
        // every engine needs a constructor with these arguments
        TimetableEngine(QObject* parent, const QVariantList& args);

    Q_SIGNALS:
        void isBusy( bool busy );
 
    protected:
	void init();
        // this virtual function is called when a new source is requested
        bool sourceRequestEvent(const QString& name);
 
        // this virtual function is called when an automatic update
        // is triggered for an existing source (ie: when a valid update
        // interval is set when requesting a source)
        bool updateSourceEvent(const QString& source);

    private Q_SLOTS:
	void networkStatusChanged( Solid::Networking::Status );

    private:
	bool mLoadedSource;
	QString mName;
	QStringList getRenfe(const QString &name);
	QStringList getFGC(const QString &name);
	QString getRenfeCode(const QString &name);
	QString getFGCCode(const QString &name);
};
 
#endif // TIMETABLE_ENGINE_H